@@ -30,7 +30,7 @@
 "*ignoreUninstalledPrograms: False",
 "*textMode:		file",
 "*textLiteral:		XScreenSaver",
-"*textFile:		/usr/X11R6/README",
+"*textFile:		",
 "*textProgram:		fortune",
 "*textURL:		http://www.livejournal.com/stats/latest-rss.bml",
 "*overlayTextForeground:	#FFFF00",
@@ -203,6 +203,10 @@
 				fiberlamp -root				    \\n\
 				boxfit -root				    \\n\
 				celtic -root				    \\n\
+				cwaves -root				    \\n\
+				m6502 -root				    \\n\
+				abstractile -root			    \\n\
+-				lcdscrub -root				    \\n\
 - default-n:			webcollage -root			    \\n\
 - default-n:  \"WebCollage (whacked)\"					      \
 				webcollage -root -filter		      \
@@ -289,6 +293,14 @@
 	   GL:			juggler3d -root				    \\n\
 	   GL:			topblock -root				    \\n\
 	   GL:			glschool -root				    \\n\
+	   GL:			glcells -root				    \\n\
+	   GL:			voronoi -root				    \\n\
+	   GL:			moebiusgears -root			    \\n\
+	   GL:			lockward -root				    \\n\
+	   GL:			cubicgrid -root				    \\n\
+	   GL:			hypnowheel -root			    \\n\
+	   GL:	\"Hypnowheel (dense)\"	hypnowheel -root -count 3 -layers 50 \\n\
+	   GL:	\"Hypnowheel (trifoil)\"	hypnowheel -root -count 3 -layers 2 -speed 9 -twist 9 -wander \\n\
 									      \
 -				xdaliclock -root -font BUILTIN3		    \\n\
 -				xplanet -vroot -wait 1 -timewarp 90000        \
@@ -340,7 +352,7 @@
 "*passwd.asterisks:              True",
 "*passwd.uname:                  True",
 "*splash.heading.label:		XScreenSaver %s",
-"*splash.body.label:		Copyright  1991-2006 by",
+"*splash.body.label:		Copyright  1991-2008 by",
 "*splash.body2.label:		Jamie Zawinski <jwz@jwz.org>",
 "*splash.demo.label:		Settings",
 "*splash.help.label:		Help",
@@ -401,4 +413,8 @@
 "*hacks.glhanoi.name:        GLHanoi",
 "*hacks.topblock.name:       TopBlock",
 "*hacks.glschool.name:       GLSchool",
+"*hacks.glcells.name:        GLCells",
+"*hacks.moebiusgears.name:   MoebiusGears",
+"*hacks.cubicgrid.name:      CubicGrid",
+"*hacks.lcdscrub.name:       LCDscrub",
 "*hacks.documentation.isInstalled: True",
